@@ -19,12 +19,18 @@ import deprecated.sphinx
         """
         This function adds *x* and *y*.
 
+        :param x: number *x*
+        :param y: number *y*
+        :return: sum = *x* + *y*
+        """,
+        """This function adds *x* and *y*.
+
         :param x: number *x*
         :param y: number *y*
         :return: sum = *x* + *y*
         """,
     ],
-    ids=["no_docstring", "short_docstring", "long_docstring"]
+    ids=["no_docstring", "short_docstring", "D213_long_docstring", "D212_long_docstring"],
 )
 def docstring(request):
     return request.param
@@ -44,9 +50,9 @@ def directive(request):
             '1.2.0',
             textwrap.dedent(
                 """\
-                     .. {directive}:: {version}
-                        {reason}
-                     """
+                .. {directive}:: {version}
+                   {reason}
+                """
             ),
         ),
         (
@@ -54,22 +60,12 @@ def directive(request):
             '1.2.0',
             textwrap.dedent(
                 """\
-                     .. {directive}:: {version}
-                     """
-            ),
-        ),
-        (
-            'A good reason',
-            None,
-            textwrap.dedent(
-                """\
-                     .. {directive}::
-                        {reason}
-                     """
+                .. {directive}:: {version}
+                """
             ),
         ),
     ],
-    ids=["reason&version", "version", "reason"]
+    ids=["reason&version", "version"],
 )
 def test_has_sphinx_docstring(docstring, directive, reason, version, expected):
     # The function:
@@ -90,10 +86,13 @@ def test_has_sphinx_docstring(docstring, directive, reason, version, expected):
     current = textwrap.dedent(foo.__doc__)
     assert current.endswith(expected)
 
-    # An empty line must separate the original docstring and the directive.
     current = current.replace(expected, '')
-    if current:
+    if docstring:
+        # An empty line must separate the original docstring and the directive.
         assert re.search("\n[ ]*\n$", current, flags=re.DOTALL)
+    else:
+        # Avoid "Explicit markup ends without a blank line" when the decorated function has no docstring
+        assert current == "\n"
 
     with warnings.catch_warnings(record=True) as warns:
         foo(1, 2)
@@ -118,9 +117,9 @@ def test_has_sphinx_docstring(docstring, directive, reason, version, expected):
             '1.2.0',
             textwrap.dedent(
                 """\
-                     .. {directive}:: {version}
-                        {reason}
-                     """
+                .. {directive}:: {version}
+                   {reason}
+                """
             ),
         ),
         (
@@ -128,22 +127,12 @@ def test_has_sphinx_docstring(docstring, directive, reason, version, expected):
             '1.2.0',
             textwrap.dedent(
                 """\
-                     .. {directive}:: {version}
-                     """
-            ),
-        ),
-        (
-            'A good reason',
-            None,
-            textwrap.dedent(
-                """\
-                     .. {directive}::
-                        {reason}
-                     """
+                .. {directive}:: {version}
+                """
             ),
         ),
     ],
-    ids=["reason&version", "version", "reason"]
+    ids=["reason&version", "version"],
 )
 def test_cls_has_sphinx_docstring(docstring, directive, reason, version, expected):
     # The class:
@@ -164,10 +153,13 @@ def test_cls_has_sphinx_docstring(docstring, directive, reason, version, expecte
     current = textwrap.dedent(Foo.__doc__)
     assert current.endswith(expected)
 
-    # An empty line must separate the original docstring and the directive.
     current = current.replace(expected, '')
-    if current:
+    if docstring:
+        # An empty line must separate the original docstring and the directive.
         assert re.search("\n[ ]*\n$", current, flags=re.DOTALL)
+    else:
+        # Avoid "Explicit markup ends without a blank line" when the decorated function has no docstring
+        assert current == "\n"
 
     with warnings.catch_warnings(record=True) as warns:
         Foo()
@@ -185,119 +177,71 @@ class MyDeprecationWarning(DeprecationWarning):
 
 
 _PARAMS = [
-    None,
-    ((), {}),
-    (('Good reason',), {}),
-    ((), {'reason': 'Good reason'}),
-    ((), {'version': '1.2.3'}),
-    ((), {'action': 'once'}),
-    ((), {'category': MyDeprecationWarning}),
+    {'version': '1.2.3'},
+    {'version': '1.2.3', 'reason': 'Good reason'},
+    {'version': '1.2.3', 'action': 'once'},
+    {'version': '1.2.3', 'category': MyDeprecationWarning},
 ]
 
 
 @pytest.fixture(scope="module", params=_PARAMS)
 def sphinx_deprecated_function(request):
-    if request.param is None:
+    kwargs = request.param
 
-        @deprecated.sphinx.deprecated
-        def foo1():
-            pass
+    @deprecated.sphinx.deprecated(**kwargs)
+    def foo1():
+        pass
 
-        return foo1
-    else:
-        args, kwargs = request.param
-
-        @deprecated.sphinx.deprecated(*args, **kwargs)
-        def foo1():
-            pass
-
-        return foo1
+    return foo1
 
 
 @pytest.fixture(scope="module", params=_PARAMS)
 def sphinx_deprecated_class(request):
-    if request.param is None:
+    kwargs = request.param
 
-        @deprecated.sphinx.deprecated
-        class Foo2(object):
-            pass
+    @deprecated.sphinx.deprecated(**kwargs)
+    class Foo2(object):
+        pass
 
-        return Foo2
-    else:
-        args, kwargs = request.param
-
-        @deprecated.sphinx.deprecated(*args, **kwargs)
-        class Foo2(object):
-            pass
-
-        return Foo2
+    return Foo2
 
 
 @pytest.fixture(scope="module", params=_PARAMS)
 def sphinx_deprecated_method(request):
-    if request.param is None:
+    kwargs = request.param
 
-        class Foo3(object):
-            @deprecated.sphinx.deprecated
-            def foo3(self):
-                pass
+    class Foo3(object):
+        @deprecated.sphinx.deprecated(**kwargs)
+        def foo3(self):
+            pass
 
-        return Foo3
-    else:
-        args, kwargs = request.param
-
-        class Foo3(object):
-            @deprecated.sphinx.deprecated(*args, **kwargs)
-            def foo3(self):
-                pass
-
-        return Foo3
+    return Foo3
 
 
 @pytest.fixture(scope="module", params=_PARAMS)
 def sphinx_deprecated_static_method(request):
-    if request.param is None:
+    kwargs = request.param
 
-        class Foo4(object):
-            @staticmethod
-            @deprecated.sphinx.deprecated
-            def foo4():
-                pass
+    class Foo4(object):
+        @staticmethod
+        @deprecated.sphinx.deprecated(**kwargs)
+        def foo4():
+            pass
 
-        return Foo4.foo4
-    else:
-        args, kwargs = request.param
-
-        class Foo4(object):
-            @staticmethod
-            @deprecated.sphinx.deprecated(*args, **kwargs)
-            def foo4():
-                pass
-
-        return Foo4.foo4
+    return Foo4.foo4
 
 
 @pytest.fixture(scope="module", params=_PARAMS)
 def sphinx_deprecated_class_method(request):
-    if request.param is None:
+    kwargs = request.param
 
-        class Foo5(object):
-            @classmethod
-            @deprecated.sphinx.deprecated
-            def foo5(cls):
-                pass
+    class Foo5(object):
+        @classmethod
+        @deprecated.sphinx.deprecated(**kwargs)
+        def foo5(cls):
+            pass
 
-        return Foo5
-    else:
-        args, kwargs = request.param
-
-        class Foo5(object):
-            @classmethod
-            @deprecated.sphinx.deprecated(*args, **kwargs)
-            def foo5(cls):
-                pass
-
-        return Foo5
+    return Foo5
 
 
 # noinspection PyShadowingNames
@@ -357,7 +301,7 @@ def test_sphinx_deprecated_class_method__warns(sphinx_deprecated_class_method):
     assert len(warns) == 1
     warn = warns[0]
     assert issubclass(warn.category, DeprecationWarning)
-    if sys.version_info >= (3, 9):
+    if (3, 9) <= sys.version_info < (3, 13):
         assert "deprecated class method" in str(warn.message)
     else:
         assert "deprecated function (or staticmethod)" in str(warn.message)
@@ -365,7 +309,10 @@ def test_sphinx_deprecated_class_method__warns(sphinx_deprecated_class_method):
 
 def test_should_raise_type_error():
     try:
-        deprecated.sphinx.deprecated(5)
+        @deprecated.sphinx.deprecated(version="4.5.6", reason=5)
+        def foo():
+            pass
+
         assert False, "TypeError not raised"
     except TypeError:
         pass
@@ -374,7 +321,7 @@ def test_should_raise_type_error():
 def test_warning_msg_has_reason():
     reason = "Good reason"
 
-    @deprecated.sphinx.deprecated(reason=reason)
+    @deprecated.sphinx.deprecated(version="4.5.6", reason=reason)
     def foo():
         pass
 
@@ -398,7 +345,7 @@ def test_warning_msg_has_version():
 
 
 def test_warning_is_ignored():
-    @deprecated.sphinx.deprecated(action='ignore')
+    @deprecated.sphinx.deprecated(version="4.5.6", action='ignore')
     def foo():
         pass
 
@@ -408,7 +355,7 @@ def test_warning_is_ignored():
 
 
 def test_specific_warning_cls_is_used():
-    @deprecated.sphinx.deprecated(category=MyDeprecationWarning)
+    @deprecated.sphinx.deprecated(version="4.5.6", category=MyDeprecationWarning)
     def foo():
         pass
 
@@ -423,3 +370,55 @@ def test_can_catch_warnings():
         warnings.simplefilter("always")
         warnings.warn("A message in a bottle", category=DeprecationWarning, stacklevel=2)
     assert len(warns) == 1
+
+
+@pytest.mark.parametrize(
+    ["reason", "expected"],
+    [
+        ("Use :function:`bar` instead", "Use `bar` instead"),
+        ("Use :py:func:`bar` instead", "Use `bar` instead"),
+    ],
+)
+def test_sphinx_syntax_trimming(reason, expected):
+    @deprecated.sphinx.deprecated(version="4.5.6", reason=reason)
+    def foo():
+        pass
+
+    with warnings.catch_warnings(record=True) as warns:
+        foo()
+    warn = warns[0]
+    assert expected in str(warn.message)
+
+
+# noinspection SpellCheckingInspection
+@pytest.mark.parametrize(
+    "reason, expected",
+    [
+        # classic examples using the default domain (Python)
+        ("Use :func:`bar` instead", "Use `bar` instead"),
+        ("Use :function:`bar` instead", "Use `bar` instead"),
+        ("Use :class:`Baz` instead", "Use `Baz` instead"),
+        ("Use :exc:`Baz` instead", "Use `Baz` instead"),
+        ("Use :exception:`Baz` instead", "Use `Baz` instead"),
+        ("Use :meth:`Baz.bar` instead", "Use `Baz.bar` instead"),
+        ("Use :method:`Baz.bar` instead", "Use `Baz.bar` instead"),
+        # other examples using a domain :
+        ("Use :py:func:`bar` instead", "Use `bar` instead"),
+        ("Use :cpp:func:`bar` instead", "Use `bar` instead"),
+        ("Use :js:func:`bar` instead", "Use `bar` instead"),
+        # the reference can have special characters:
+        ("Use :func:`~pkg.mod.bar` instead", "Use `~pkg.mod.bar` instead"),
+        # edge cases:
+        ("Use :r:`` instead", "Use `` instead"),
+        ("Use :d:r:`` instead", "Use `` instead"),
+        ("Use :r:`foo` instead", "Use `foo` instead"),
+        ("Use :d:r:`foo` instead", "Use `foo` instead"),
+        ("Use r:`bad` instead", "Use r:`bad` instead"),
+        ("Use ::`bad` instead", "Use ::`bad` instead"),
+        ("Use :::`bad` instead", "Use :::`bad` instead"),
+    ],
+)
+def test_get_deprecated_msg(reason, expected):
+    adapter = deprecated.sphinx.SphinxAdapter("deprecated", reason=reason, version="1")
+    actual = adapter.get_deprecated_msg(lambda: None, None)
+    assert expected in actual
